#include "os.h"
#include "kheap.h"
#include "task.h"
#include "initrd.h"
#include "syscall.h"

extern ULONG file_data_start;
extern ULONG file_data_end;

extern page_directory_t* current_directory;
extern task_t* current_task;
extern task_t* ready_queue;
extern tss_entry_t tss_entry;
extern heap_t* kheap;

ULONG ramdisk_start;              // at kernel heap
ULONG address_user  = 0x00400000; // begin of user space
UCHAR address_TEST[4096];

int main()
{
    k_clear_screen(); settextcolor(14,0);
    printformat("PrettyOS [Version 0.1.0001]\n(C) 2009 henkessoft.de\n\n");
    gdt_install();
    idt_install();
    isrs_install();
    irq_install();
    initODA();
    timer_install();
    keyboard_install();
    paging_install();

    kheap = create_heap(KHEAP_START, KHEAP_START+KHEAP_INITIAL_SIZE, KHEAP_MAX, 1, 0); // SV and RW

    tasking_install();
    sti();

    ramdisk_start = k_malloc(0x100000,0,0);
    k_memcpy((void*)ramdisk_start, &file_data_start, (ULONG)&file_data_end - (ULONG)&file_data_start); //process.asm
    fs_root = install_initrd(ramdisk_start);

    // search the content of files <- data from outside "loaded" via incbin ...
    settextcolor(15,0);
    ULONG i = 0;
    struct dirent* node = 0;
    while( (node = readdir_fs(fs_root, i)) != 0)
    {
        fs_node_t* fsnode = finddir_fs(fs_root, node->name);

        if((fsnode->flags & 0x7) == FS_DIRECTORY)
        {
            printformat("<DIR> %s\n",node->name);
        }
        else
        {
            UCHAR buf[4096];
            ULONG sz = read_fs(fsnode, 0, fsnode->length, buf);
            printformat("%d \t%s\n",sz,node->name);

            ULONG j;
            for(j=0; j<sz; ++j)
            {
                if( k_strcmp(node->name,"Test-Programm")==0 )
                {
                    address_TEST[j] = buf[j];
                }
            }
        }
        ++i;
    }
    printformat("\n\n");
    //Test-Programm ==> user space
    k_memcpy((void*)address_user, address_TEST+0x60, 0xbb); // ELF LOAD: Offset VADDR FileSize

    pODA->ts_flag = 1; // enable task_switching
    create_task ((void*)0x400000,3); //user space beginning

    while(TRUE) { /*  */ }
    return 0;
}
